import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import logging

logging.basicConfig(level=logging.INFO)

def create_dataset(dataset, time_step=1):
    logging.info("Creating dataset with time_step=%d", time_step)
    X, Y = [], []
    for i in range(len(dataset) - time_step - 1):
        a = dataset[i:(i + time_step), 0]
        X.append(a)
        Y.append(dataset[i + time_step, 0])
    return np.array(X), np.array(Y)

def predict(model, test_data, scaler, time_step=60):
    logging.info("Starting prediction")
    X_test, y_test = create_dataset(test_data.values, time_step)
    X_test = X_test.reshape(X_test.shape[0], X_test.shape[1], 1)
    predictions = model.predict(X_test)
    predictions = scaler.inverse_transform(predictions)
    logging.info("Prediction completed")
    return predictions, y_test

def plot_predictions(train_data, test_data, predictions):
    logging.info("Plotting predictions")
    train = train_data['Close']
    valid = test_data['Close']
    valid = pd.DataFrame(valid)
    valid['Predictions'] = predictions

    plt.figure(figsize=(16, 8))
    plt.title('Model')
    plt.xlabel('Date')
    plt.ylabel('Close Price KRW (₩)')
    plt.plot(train.index, train, label='Train')
    plt.plot(valid.index, valid[['Close', 'Predictions']])
    plt.legend(['Train', 'Val', 'Predictions'], loc='lower right')
    plt.show()

def main():
    from model_training import train_model
    from data_preprocessing import preprocess_data
    from data_collection import update_data

    try:
        logging.info("Updating data")
        stock_data, news_data, financial_data, economic_data = update_data()

        logging.info("Preprocessing data")
        (train_data, test_data), scaler = preprocess_data(stock_data, financial_data, news_data, economic_data)

        logging.info("Training model")
        model = train_model(train_data['Close'])

        logging.info("Making predictions")
        predictions, y_test = predict(model, test_data['Close'], scaler)

        logging.info("Plotting results")
        plot_predictions(train_data, test_data, predictions)

    except Exception as e:
        logging.error(f"An error occurred: {e}")

if __name__ == '__main__':
    main()
